----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: tb_scalp_rx_side.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Testbench for scalp_rx_side.
--
-- Last update: 2020-02-13
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;
-- use ieee.math_real.uniform;
-- use ieee.math_real.floor;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.aurora_axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

entity tb_scalp_rx_side is

end entity tb_scalp_rx_side;

architecture testbench of tb_scalp_rx_side is

    constant C_CLK_PERIOD                  : time                                                            := 10 ns;
    constant C_SCALP_NEIGHBORS_VECTOR_SIZE : integer range 0 to 255                                          := 6;
    constant C_SCALP_SCHEDULER_STRATEGY    : string                                                          := "RR";
    constant C_SCALP_SCHEDULER_OUTPUT_REG  : boolean                                                         := true;
    -- scalp_sp_to_axis
    constant C_SCALP_PACKET_PAYLOAD_SIZE   : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE            := 8;
    constant C_SCALP_RANDOM_READY          : boolean                                                         := true;
    constant C_SCALP_DELAY_SIZE_0          : integer range 0 to 255                                          := 5;
    constant C_SCALP_DELAY_SIZE_1          : integer range 0 to 255                                          := 10;
    constant C_SCALP_DELAY_SIZE_2          : integer range 0 to 255                                          := 15;
    constant C_SCALP_DELAY_SIZE_3          : integer range 0 to 255                                          := 20;
    constant C_SCALP_DELAY_SIZE_4          : integer range 0 to 255                                          := 25;
    constant C_SCALP_DELAY_SIZE_5          : integer range 0 to 255                                          := 30;
    constant C_SCALP_BOOKING_VECTOR        : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := "001010";

    -- Scalp Packet Headers
    constant C_SP_HEADER_0 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 1,
             YxD     => 2,
             ZxD     => 3),
            SrcAddrxD =>
            (XxD     => 4,
             YxD     => 5,
             ZxD     => 6),
            TypexD   => 1,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);

    constant C_SP_HEADER_1 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 7,
             YxD     => 8,
             ZxD     => 9),
            SrcAddrxD =>
            (XxD     => 10,
             YxD     => 11,
             ZxD     => 12),
            TypexD   => 0,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);

    constant C_SP_HEADER_2 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 13,
             YxD     => 14,
             ZxD     => 15),
            SrcAddrxD =>
            (XxD     => 16,
             YxD     => 17,
             ZxD     => 18),
            TypexD   => 1,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);

    constant C_SP_HEADER_3 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 19,
             YxD     => 20,
             ZxD     => 21),
            SrcAddrxD =>
            (XxD     => 22,
             YxD     => 23,
             ZxD     => 24),
            TypexD   => 0,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);

    constant C_SP_HEADER_4 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 25,
             YxD     => 26,
             ZxD     => 27),
            SrcAddrxD =>
            (XxD     => 28,
             YxD     => 29,
             ZxD     => 30),
            TypexD   => 1,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);

    constant C_SP_HEADER_5 : t_scalp_packet_header
        := (DstAddrxD =>
            (XxD     => 31,
             YxD     => 32,
             ZxD     => 33),
            SrcAddrxD =>
            (XxD     => 34,
             YxD     => 35,
             ZxD     => 36),
            TypexD   => 0,
            LengthxD => C_SCALP_PACKET_PAYLOAD_SIZE);
    -- Scalp Packet Payloads
    constant C_SP_PAYLOAD_0 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#0f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#01223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#02334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    constant C_SP_PAYLOAD_1 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#1f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#11223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#12334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    constant C_SP_PAYLOAD_2 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#2f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#21223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#22334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    constant C_SP_PAYLOAD_3 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#3f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#31223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#32334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    constant C_SP_PAYLOAD_4 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#4f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#41223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#42334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    constant C_SP_PAYLOAD_5 : t_scalp_packet_payload(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))
        := (0 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5abbccdd#, (C_BYTE_SIZE * 4))),
             IdxD   => 0),
            1 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5bccddee#, (C_BYTE_SIZE * 4))),
             IdxD   => 1),
            2 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5cddeeff#, (C_BYTE_SIZE * 4))),
             IdxD   => 2),
            3 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5deeff11#, (C_BYTE_SIZE * 4))),
             IdxD   => 3),
            4 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5eff1122#, (C_BYTE_SIZE * 4))),
             IdxD   => 4),
            5 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#5f112233#, (C_BYTE_SIZE * 4))),
             IdxD   => 5),
            6 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#51223344#, (C_BYTE_SIZE * 4))),
             IdxD   => 6),
            7 =>
            (WordxD => std_ulogic_vector(to_unsigned(16#52334455#, (C_BYTE_SIZE * 4))),
             IdxD   => 7));

    component scalp_sp_to_axis is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpPacketxDI      : in  t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSI : in  std_ulogic;
            ScalpAxism2sxDO     : out t_axi4m2s;
            ScalpAxiss2mxDI     : in  t_axi4s2m);
    end component scalp_sp_to_axis;

    component scalp_axis_to_sp is
        generic (
            C_SCALP_PACKET_PAYLOAD_SIZE : integer range 1 to C_SCALP_PACKET_LENGTH_RANGE_VALUE;
            C_SCALP_RANDOM_READY        : boolean);
        port (
            SysClkxCI           : in  std_ulogic;
            SysRstxRNAI         : in  std_ulogic;
            ScalpAxism2sxDI     : in  t_axi4m2s;
            ScalpAxiss2mxDO     : out t_axi4s2m;
            ScalpPacketxDO      : out t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1)));
            ScalpPacketValidxSO : out std_ulogic);
    end component scalp_axis_to_sp;

    component scalp_rx_side is
        generic (
            C_SCALP_NEIGHBORS_VECTOR_SIZE : integer range 0 to 255;
            C_SCALP_SCHEDULER_STRATEGY    : string;
            C_SCALP_SCHEDULER_OUTPUT_REG  : boolean);
        port (
            -- System Clock and Reset
            SysClkxCI            : in  std_ulogic;
            SysRstxRNAI          : in  std_ulogic;
            -- Scheduler Ack Vector
            SchedulerAckxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Booking Vector
            BookingVectorxDI     : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Axi Stream
            -- Output Side
            TXAxi4M2SLinkxDO     : out t_axi4m2s;
            TXAxi4S2MLinkxDI     : in  t_axi4s2m;
            -- Input Side
            RXAxi4M2SLinkxDI     : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            RXAxi4S2MLinkxDO     : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Debug Interface
            BookingIndexxDO      : out integer range 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1);
            BookingIndexValidxSO : out std_ulogic);
    end component scalp_rx_side;

    -- Sys Clk and Sys nRst
    signal SysClkxC              : std_ulogic                                                      := '0';
    signal SysRstxRNA            : std_ulogic                                                      := '0';
    -- End of Simulation
    signal EndSimxS              : boolean                                                         := false;
    signal SchedAckxS            : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal SPValidEndxS          : std_ulogic                                                      := '0';
    -- scalp_rx_side
    signal SchedulerAckxD        : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal BookingVectorxD       : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal NewBookingVectorxD    : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal TXAxi4M2SLinkxD       : t_axi4m2s                                                       := C_NO_AXI4_M2S;
    signal TXAxi4S2MLinkxD       : t_axi4s2m                                                       := C_NO_AXI4_S2M;
    signal RXAxi4M2SLinkxD       : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => C_NO_AXI4_M2S);
    signal RXAxi4S2MLinkxD       : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)  := (others => C_NO_AXI4_S2M);
    signal BookingIndexxD        : integer range 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1)          := 0;
    signal BookingIndexValidxS   : std_ulogic                                                      := '0';
    -- Scalp Packets
    signal ScalpPacketValidxD    : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => '0');
    signal ScalpPacketValidEndxD : std_ulogic                                                      := '0';

    -- Packet 0
    signal ScalpPacket0xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_0,
         SpPayloadxD => C_SP_PAYLOAD_0);
    -- Packet 1
    signal ScalpPacket1xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_1,
         SpPayloadxD => C_SP_PAYLOAD_1);
    -- Packet 2
    signal ScalpPacket2xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_2,
         SpPayloadxD => C_SP_PAYLOAD_2);
    -- Packet 3
    signal ScalpPacket3xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_3,
         SpPayloadxD => C_SP_PAYLOAD_3);
    -- Packet 4
    signal ScalpPacket4xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_4,
         SpPayloadxD => C_SP_PAYLOAD_4);
    -- Packet 5
    signal ScalpPacket5xD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_SP_HEADER_5,
         SpPayloadxD => C_SP_PAYLOAD_5);
    -- Packet TX Side
    signal ScalpPacketEndxD : t_scalp_packet(SpPayloadxD(0 to (C_SCALP_PACKET_PAYLOAD_SIZE - 1))) :=
        (SpHeaderxD  => C_NO_SCALP_PACKET_HEADER,
         SpPayloadxD => (others => C_NO_SCALP_PACKET_WORD));

begin  -- architecture testbench

    ScalpSpToAxis0xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket0xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(0),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(0),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(0));

    ScalpSpToAxis1xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket1xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(1),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(1),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(1));

    ScalpSpToAxis2xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket2xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(2),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(2),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(2));

    ScalpSpToAxis3xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket3xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(3),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(3),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(3));

    ScalpSpToAxis4xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket4xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(4),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(4),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(4));

    ScalpSpToAxis5xI : entity work.scalp_sp_to_axis
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpPacketxDI      => ScalpPacket5xD,
            ScalpPacketValidxSI => ScalpPacketValidxD(5),
            ScalpAxism2sxDO     => RXAxi4M2SLinkxD(5),
            ScalpAxiss2mxDI     => RXAxi4S2MLinkxD(5));

    ScalpRXSidexI : entity work.scalp_rx_side
        generic map (
            C_SCALP_NEIGHBORS_VECTOR_SIZE => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_SCHEDULER_STRATEGY    => C_SCALP_SCHEDULER_STRATEGY,
            C_SCALP_SCHEDULER_OUTPUT_REG  => C_SCALP_SCHEDULER_OUTPUT_REG)
        port map (
            -- System Clock and Reset
            SysClkxCI            => SysClkxC,
            SysRstxRNAI          => SysRstxRNA,
            -- Scheduler Ack Vector
            SchedulerAckxDO      => SchedulerAckxD,
            -- Booking Vector
            BookingVectorxDI     => BookingVectorxD,
            -- Axi Stream
            -- Output Side
            TXAxi4M2SLinkxDO     => TXAxi4M2SLinkxD,
            TXAxi4S2MLinkxDI     => TXAxi4S2MLinkxD,
            -- Input Side
            RXAxi4M2SLinkxDI     => RXAxi4M2SLinkxD,
            RXAxi4S2MLinkxDO     => RXAxi4S2MLinkxD,
            -- Debug Interface
            BookingIndexxDO      => BookingIndexxD,
            BookingIndexValidxSO => BookingIndexValidxS);

    ScalpAxisToSpxI : entity work.scalp_axis_to_sp
        generic map (
            C_SCALP_PACKET_PAYLOAD_SIZE => C_SCALP_PACKET_PAYLOAD_SIZE,
            C_SCALP_RANDOM_READY        => C_SCALP_RANDOM_READY)
        port map (
            SysClkxCI           => SysClkxC,
            SysRstxRNAI         => SysRstxRNA,
            ScalpAxism2sxDI     => TXAxi4M2SLinkxD,
            ScalpAxiss2mxDO     => TXAxi4S2MLinkxD,
            ScalpPacketxDO      => ScalpPacketEndxD,
            ScalpPacketValidxSO => ScalpPacketValidEndxD);

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB
        SysClkxAS : SysClkxC   <= not SysClkxC after (C_CLK_PERIOD / 2);
        SysRstxAS : SysRstxRNA <= '1'          after (C_CLK_PERIOD * 10);
    end block AsyncStatementsxB;

    EndSimxP : process is
    begin  -- process EndSimxP
        if EndSimxS = true then
            assert false report "End of simulation!" severity failure;
        end if;

        wait until rising_edge(SysClkxC);
    end process EndSimxP;

    WaitSchedulerAckxG : for i in 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) generate
        WaitSchedulerAckxP : process is
        begin  -- process WaitSchedulerAckxP
            wait until SchedulerAckxD(i) = '1';
            wait until SchedulerAckxD(i) = '0';

            SchedAckxS(i) <= '1';
            wait;
        end process WaitSchedulerAckxP;
    end generate WaitSchedulerAckxG;

    WaitScalpPacketValidEndxP : process is
    begin  -- process WaitScalpPacketValidEndxP
        for i in 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) loop
            wait until ScalpPacketValidEndxD = '1';
            wait until ScalpPacketValidEndxD = '0';
        end loop;  -- i

        SPValidEndxS <= '1';
        wait;
    end process WaitScalpPacketValidEndxP;

    ScalpRXSidexP : process is
    begin  -- process ScalpRXSidexP

        -- TXAxi4S2MLinkxD.ReadyxS <= '0';

        wait until (SysRstxRNA = '1');
        wait until rising_edge(SysClkxC);
        wait until rising_edge(SysClkxC);

        ScalpPacketValidxD <= (others => '0');
        BookingVectorxD    <= (others => '0');

        -- TXAxi4S2MLinkxD.ReadyxS <= '1';

        for i in 0 to 0 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxD <= "111111";
        BookingVectorxD    <= "111111";

        for i in 0 to 0 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        ScalpPacketValidxD <= (others => '0');

        wait until
            ((SchedAckxS(0) = '1') and
             (SchedAckxS(1) = '1') and
             (SchedAckxS(2) = '1') and
             (SchedAckxS(3) = '1') and
             (SchedAckxS(4) = '1') and
             (SchedAckxS(5) = '1') and
             SPValidEndxS = '1');

        for i in 0 to 9 loop
            wait until rising_edge(SysClkxC);
        end loop;  -- i

        EndSimxS <= true;

    end process ScalpRXSidexP;

end architecture testbench;
